@@ -11,11 +11,13 @@ typedef void (*GC_for_each_struct_proc)(void *p);
 typedef void (*GC_print_tagged_value_proc)(const char *prefix, 
 					   void *v, uintptr_t diff, int max_w,
 					   const char *suffix);
+typedef int (*GC_print_traced_filter_proc)(void *p);
 
 GC2_EXTERN void GC_dump_with_traces(int flags,
 				    GC_get_type_name_proc get_type_name,
 				    GC_for_each_found_proc for_each_found,
 				    short min_trace_for_tag, short max_trace_for_tag,
+                                    GC_print_traced_filter_proc print_traced_filter,
 				    GC_print_tagged_value_proc print_tagged_value,
 				    int path_length_limit,
                                     GC_for_each_struct_proc for_each_struct);
