@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2015 PLT Design Inc.
+  Copyright (c) 2004-2016 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -34,10 +34,10 @@
 READ_ONLY Scheme_Object scheme_true[1];
 READ_ONLY Scheme_Object scheme_false[1];
 
-READ_ONLY Scheme_Object *scheme_not_prim;
-READ_ONLY Scheme_Object *scheme_eq_prim;
-READ_ONLY Scheme_Object *scheme_eqv_prim;
-READ_ONLY Scheme_Object *scheme_equal_prim;
+READ_ONLY Scheme_Object *scheme_not_proc;
+READ_ONLY Scheme_Object *scheme_eq_proc;
+READ_ONLY Scheme_Object *scheme_eqv_proc;
+READ_ONLY Scheme_Object *scheme_equal_proc;
 
 /* locals */
 static Scheme_Object *not_prim (int argc, Scheme_Object *argv[]);
@@ -48,6 +48,7 @@ static Scheme_Object *equal_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *equalish_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *chaperone_p (int argc, Scheme_Object *argv[]);
 static Scheme_Object *impersonator_p (int argc, Scheme_Object *argv[]);
+static Scheme_Object *procedure_impersonator_star_p (int argc, Scheme_Object *argv[]);
 static Scheme_Object *chaperone_of (int argc, Scheme_Object *argv[]);
 static Scheme_Object *impersonator_of (int argc, Scheme_Object *argv[]);
 
@@ -82,13 +83,13 @@ void scheme_init_bool (Scheme_Env *env)
 {
   Scheme_Object *p;
 
-  REGISTER_SO(scheme_not_prim);
-  REGISTER_SO(scheme_eq_prim);
-  REGISTER_SO(scheme_eqv_prim);
-  REGISTER_SO(scheme_equal_prim);
+  REGISTER_SO(scheme_not_proc);
+  REGISTER_SO(scheme_eq_proc);
+  REGISTER_SO(scheme_eqv_proc);
+  REGISTER_SO(scheme_equal_proc);
 
   p = scheme_make_folding_prim(not_prim, "not", 1, 1, 1);
-  scheme_not_prim = p;
+  scheme_not_proc = p;
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_UNARY_INLINED
                                                             | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("not", p, env);
@@ -101,19 +102,19 @@ void scheme_init_bool (Scheme_Env *env)
   p = scheme_make_folding_prim(eq_prim, "eq?", 2, 2, 1);
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_BINARY_INLINED
                                                             | SCHEME_PRIM_IS_OMITABLE);
-  scheme_eq_prim = p;
+  scheme_eq_proc = p;
   scheme_add_global_constant("eq?", p, env);
 
   p = scheme_make_folding_prim(eqv_prim, "eqv?", 2, 2, 1);
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_BINARY_INLINED
                                                             | SCHEME_PRIM_IS_OMITABLE);
-  scheme_eqv_prim = p;
-  scheme_add_global_constant("eqv?", scheme_eqv_prim, env);
+  scheme_eqv_proc = p;
+  scheme_add_global_constant("eqv?", scheme_eqv_proc, env);
   
   p = scheme_make_prim_w_arity(equal_prim, "equal?", 2, 2);
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_BINARY_INLINED);
-  scheme_equal_prim = p;
-  scheme_add_global_constant("equal?", scheme_equal_prim, env);
+  scheme_equal_proc = p;
+  scheme_add_global_constant("equal?", scheme_equal_proc, env);
 
   scheme_add_global_constant("equal?/recur", 
                              scheme_make_prim_w_arity(equalish_prim, "equal?/recur", 3, 3), 
@@ -128,6 +129,9 @@ void scheme_init_bool (Scheme_Env *env)
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_UNARY_INLINED
                                                             | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("impersonator?", p, env);
+  p = scheme_make_immed_prim(procedure_impersonator_star_p, "procedure-impersonator*?", 1, 1);
+  SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_OMITABLE);
+  scheme_add_global_constant("procedure-impersonator*?", p, env);
 
   scheme_add_global_constant("chaperone-of?",
                              scheme_make_prim_w_arity(chaperone_of, "chaperone-of?", 2, 2),
@@ -575,6 +579,8 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
 
   if (eql->for_chaperone 
       && SCHEME_CHAPERONEP(obj2)
+      && (!(SCHEME_CHAPERONE_FLAGS((Scheme_Chaperone *)obj2) & SCHEME_CHAPERONE_IS_IMPERSONATOR)
+          || (eql->for_chaperone > 1))
       && scheme_is_noninterposing_chaperone(obj2)) {
     obj2 = ((Scheme_Chaperone *)obj2)->prev;
     goto top_after_next;
@@ -605,20 +611,22 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
   t2 = SCHEME_TYPE(obj2);
 
   if (NOT_SAME_TYPE(t1, t2)) {
+    if (t1 == scheme_hash_tree_indirection_type) {
+      obj1 = (Scheme_Object *)scheme_hash_tree_resolve_placeholder((Scheme_Hash_Tree *)obj1);
+      goto top_after_next;
+    }
+    if (t2 == scheme_hash_tree_indirection_type) {
+      obj2 = (Scheme_Object *)scheme_hash_tree_resolve_placeholder((Scheme_Hash_Tree *)obj2);
+      goto top_after_next;
+    }
     if (!eql->for_chaperone) {
       if (SCHEME_CHAPERONEP(obj1)) {
         obj1 = ((Scheme_Chaperone *)obj1)->val;
         goto top_after_next;
-      } else if (t1 == scheme_hash_tree_indirection_type) {
-        obj1 = (Scheme_Object *)scheme_hash_tree_resolve_placeholder((Scheme_Hash_Tree *)obj1);
-        goto top_after_next;
       }
       if (SCHEME_CHAPERONEP(obj2)) {
         obj2 = ((Scheme_Chaperone *)obj2)->val;
         goto top_after_next;
-      } else if (t2 == scheme_hash_tree_indirection_type) {
-        obj2 = (Scheme_Object *)scheme_hash_tree_resolve_placeholder((Scheme_Hash_Tree *)obj2);
-        goto top_after_next;
       }
     }
     return 0;
@@ -985,6 +993,21 @@ static Scheme_Object *impersonator_p(int argc, Scheme_Object *argv[])
   return (SCHEME_CHAPERONEP(argv[0]) ? scheme_true : scheme_false);
 }
 
+/* Was this value created with `impersonate-procedure*` or `chaperone-procedure*`? */
+static Scheme_Object *procedure_impersonator_star_p(int argc, Scheme_Object *argv[])
+{
+  Scheme_Vector *redirects;
+  if (SCHEME_CHAPERONEP(argv[0])) {
+    redirects = (Scheme_Vector *)(((Scheme_Chaperone *)(argv[0]))->redirects);
+    if ((SCHEME_VEC_SIZE(redirects) % 2 == 1) /* odd size => procedure chaperone */
+        && ((SCHEME_VEC_SIZE(redirects) == 5) /* size 5 => we are a chap/imp* */
+            || SCHEME_IMMUTABLEP(redirects))) { /* immutable => chap/imp* in our ancestry */
+      return scheme_true;
+    }
+  }
+  return scheme_false;
+}
+
 static Scheme_Object *chaperone_of(int argc, Scheme_Object *argv[])
 {
   return (scheme_chaperone_of(argv[0], argv[1]) ? scheme_true : scheme_false);
