@@ -299,6 +299,7 @@ Scheme_Env *scheme_engine_instance_init()
   scheme_init_logging_once();
 
   scheme_init_compenv_symbol();
+  scheme_init_param_symbol();
 
 #if defined(MZ_PLACES_WAITPID)
   scheme_places_start_child_signal_handler();
@@ -1622,7 +1623,7 @@ void scheme_shadow(Scheme_Env *env, Scheme_Object *n, Scheme_Object *val, int as
   /* If the binding is a rename transformer, also install 
      a mapping */
   if (scheme_is_binding_rename_transformer(val))
-    scheme_add_binding_copy(id, scheme_rename_transformer_id(val), scheme_env_phase(env));
+    scheme_add_binding_copy(id, scheme_rename_transformer_id(val, NULL), scheme_env_phase(env));
 }
 
 static void install_one_binding_name(Scheme_Hash_Table *bt, Scheme_Object *name, Scheme_Object *id, Scheme_Env *benv)
@@ -2120,6 +2121,7 @@ static Scheme_Object *do_variable_namespace(const char *who, int tl, int argc, S
     /* return env directly; need to set up  */
     if (!env->phase && env->module)
       scheme_prep_namespace_rename(env);
+    env->interactive_bindings = 1;
   } else {
     /* new namespace: */
     Scheme_Env *new_env;
@@ -2359,7 +2361,7 @@ do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int r
     
     v = SCHEME_PTR_VAL(v);
     if (scheme_is_rename_transformer(v)) {
-      sym = scheme_transfer_srcloc(scheme_rename_transformer_id(v), sym);
+      sym = scheme_transfer_srcloc(scheme_rename_transformer_id(v, NULL), sym);
       renamed = 1;
       menv = NULL;
       SCHEME_USE_FUEL(1);
@@ -2982,7 +2984,7 @@ rename_transformer_target(int argc, Scheme_Object *argv[])
   if (!scheme_is_rename_transformer(argv[0]))
     scheme_wrong_contract("rename-transformer-target", "rename-transformer?", 0, argc, argv);
 
-  return scheme_rename_transformer_id(argv[0]);
+  return scheme_rename_transformer_id(argv[0], NULL);
 }
 
 static Scheme_Object *
