@@ -1,4 +1,4 @@
- Scheme_Object* scheme_rtcall_siS_s(const char *who, int src_type, prim_siS_s f, Scheme_Object* g235, int g236, Scheme_Object** g237)
+ Scheme_Object* scheme_rtcall_siS_s(const char *who, int src_type, prim_siS_s f, Scheme_Object* g238, int g239, Scheme_Object** g240)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -13,9 +13,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g235;
-    future->arg_i1 = g236;
-    future->arg_S2 = g237;
+      future->arg_s0 = g238;
+    future->arg_i1 = g239;
+    future->arg_S2 = g240;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -25,7 +25,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_iSs_s(const char *who, int src_type, prim_iSs_s f, int g238, Scheme_Object** g239, Scheme_Object* g240)
+ Scheme_Object* scheme_rtcall_iSs_s(const char *who, int src_type, prim_iSs_s f, int g241, Scheme_Object** g242, Scheme_Object* g243)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -40,9 +40,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g238;
-    future->arg_S1 = g239;
-    future->arg_s2 = g240;
+      future->arg_i0 = g241;
+    future->arg_S1 = g242;
+    future->arg_s2 = g243;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -52,7 +52,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_s_s(const char *who, int src_type, prim_s_s f, Scheme_Object* g241)
+ Scheme_Object* scheme_rtcall_s_s(const char *who, int src_type, prim_s_s f, Scheme_Object* g244)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -67,8 +67,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g241;
-  send_special_result(future, g241);
+      future->arg_s0 = g244;
+  send_special_result(future, g244);
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
   future = fts->thread->current_ft;
@@ -77,7 +77,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_n_s(const char *who, int src_type, prim_n_s f, Scheme_Native_Lambda* g242)
+ Scheme_Object* scheme_rtcall_n_s(const char *who, int src_type, prim_n_s f, Scheme_Native_Lambda* g245)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -92,7 +92,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_n0 = g242;
+      future->arg_n0 = g245;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -127,7 +127,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_ss_s(const char *who, int src_type, prim_ss_s f, Scheme_Object* g243, Scheme_Object* g244)
+ Scheme_Object* scheme_rtcall_ss_s(const char *who, int src_type, prim_ss_s f, Scheme_Object* g246, Scheme_Object* g247)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -142,8 +142,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g243;
-    future->arg_s1 = g244;
+      future->arg_s0 = g246;
+    future->arg_s1 = g247;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -153,7 +153,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_ssi_s(const char *who, int src_type, prim_ssi_s f, Scheme_Object* g245, Scheme_Object* g246, int g247)
+ Scheme_Object* scheme_rtcall_ssi_s(const char *who, int src_type, prim_ssi_s f, Scheme_Object* g248, Scheme_Object* g249, int g250)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -168,9 +168,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g245;
-    future->arg_s1 = g246;
-    future->arg_i2 = g247;
+      future->arg_s0 = g248;
+    future->arg_s1 = g249;
+    future->arg_i2 = g250;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -180,7 +180,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_tt_s(const char *who, int src_type, prim_tt_s f, const Scheme_Object* g248, const Scheme_Object* g249)
+ Scheme_Object* scheme_rtcall_tt_s(const char *who, int src_type, prim_tt_s f, const Scheme_Object* g251, const Scheme_Object* g252)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -195,8 +195,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_t0 = g248;
-    future->arg_t1 = g249;
+      future->arg_t0 = g251;
+    future->arg_t1 = g252;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -206,7 +206,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- MZ_MARK_STACK_TYPE scheme_rtcall_ss_m(const char *who, int src_type, prim_ss_m f, Scheme_Object* g250, Scheme_Object* g251)
+ MZ_MARK_STACK_TYPE scheme_rtcall_ss_m(const char *who, int src_type, prim_ss_m f, Scheme_Object* g253, Scheme_Object* g254)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -221,8 +221,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g250;
-    future->arg_s1 = g251;
+      future->arg_s0 = g253;
+    future->arg_s1 = g254;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -232,7 +232,7 @@
   
   return retval;
 }
- Scheme_Object* scheme_rtcall_Sl_s(const char *who, int src_type, prim_Sl_s f, Scheme_Object** g252, intptr_t g253)
+ Scheme_Object* scheme_rtcall_Sl_s(const char *who, int src_type, prim_Sl_s f, Scheme_Object** g255, intptr_t g256)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -247,8 +247,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_S0 = g252;
-    future->arg_l1 = g253;
+      future->arg_S0 = g255;
+    future->arg_l1 = g256;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -258,7 +258,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_l_s(const char *who, int src_type, prim_l_s f, intptr_t g254)
+ Scheme_Object* scheme_rtcall_l_s(const char *who, int src_type, prim_l_s f, intptr_t g257)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -273,7 +273,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_l0 = g254;
+      future->arg_l0 = g257;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -283,7 +283,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_bsi_v(const char *who, int src_type, prim_bsi_v f, Scheme_Bucket* g255, Scheme_Object* g256, int g257)
+ void scheme_rtcall_bsi_v(const char *who, int src_type, prim_bsi_v f, Scheme_Bucket* g258, Scheme_Object* g259, int g260)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -298,9 +298,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_b0 = g255;
-    future->arg_s1 = g256;
-    future->arg_i2 = g257;
+      future->arg_b0 = g258;
+    future->arg_s1 = g259;
+    future->arg_i2 = g260;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -310,7 +310,7 @@
   
   
 }
- void scheme_rtcall_iiS_v(const char *who, int src_type, prim_iiS_v f, int g258, int g259, Scheme_Object** g260)
+ void scheme_rtcall_iiS_v(const char *who, int src_type, prim_iiS_v f, int g261, int g262, Scheme_Object** g263)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -325,9 +325,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g258;
-    future->arg_i1 = g259;
-    future->arg_S2 = g260;
+      future->arg_i0 = g261;
+    future->arg_i1 = g262;
+    future->arg_S2 = g263;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -337,7 +337,7 @@
   
   
 }
- void scheme_rtcall_ss_v(const char *who, int src_type, prim_ss_v f, Scheme_Object* g261, Scheme_Object* g262)
+ void scheme_rtcall_ss_v(const char *who, int src_type, prim_ss_v f, Scheme_Object* g264, Scheme_Object* g265)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -352,8 +352,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g261;
-    future->arg_s1 = g262;
+      future->arg_s0 = g264;
+    future->arg_s1 = g265;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -363,7 +363,7 @@
   
   
 }
- void scheme_rtcall_b_v(const char *who, int src_type, prim_b_v f, Scheme_Bucket* g263)
+ void scheme_rtcall_b_v(const char *who, int src_type, prim_b_v f, Scheme_Bucket* g266)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -378,7 +378,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_b0 = g263;
+      future->arg_b0 = g266;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -388,7 +388,7 @@
   
   
 }
- Scheme_Object* scheme_rtcall_sl_s(const char *who, int src_type, prim_sl_s f, Scheme_Object* g264, intptr_t g265)
+ Scheme_Object* scheme_rtcall_sl_s(const char *who, int src_type, prim_sl_s f, Scheme_Object* g267, intptr_t g268)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -403,8 +403,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g264;
-    future->arg_l1 = g265;
+      future->arg_s0 = g267;
+    future->arg_l1 = g268;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -414,7 +414,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_iS_s(const char *who, int src_type, prim_iS_s f, int g266, Scheme_Object** g267)
+ Scheme_Object* scheme_rtcall_iS_s(const char *who, int src_type, prim_iS_s f, int g269, Scheme_Object** g270)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -429,8 +429,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g266;
-    future->arg_S1 = g267;
+      future->arg_i0 = g269;
+    future->arg_S1 = g270;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -440,7 +440,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_S_s(const char *who, int src_type, prim_S_s f, Scheme_Object** g268)
+ Scheme_Object* scheme_rtcall_S_s(const char *who, int src_type, prim_S_s f, Scheme_Object** g271)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -455,7 +455,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_S0 = g268;
+      future->arg_S0 = g271;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -465,7 +465,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_s_v(const char *who, int src_type, prim_s_v f, Scheme_Object* g269)
+ void scheme_rtcall_s_v(const char *who, int src_type, prim_s_v f, Scheme_Object* g272)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -480,8 +480,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g269;
-  send_special_result(future, g269);
+      future->arg_s0 = g272;
+  send_special_result(future, g272);
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
   future = fts->thread->current_ft;
@@ -490,7 +490,7 @@
   
   
 }
- Scheme_Object* scheme_rtcall_iSi_s(const char *who, int src_type, prim_iSi_s f, int g270, Scheme_Object** g271, int g272)
+ Scheme_Object* scheme_rtcall_iSi_s(const char *who, int src_type, prim_iSi_s f, int g273, Scheme_Object** g274, int g275)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -505,9 +505,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g270;
-    future->arg_S1 = g271;
-    future->arg_i2 = g272;
+      future->arg_i0 = g273;
+    future->arg_S1 = g274;
+    future->arg_i2 = g275;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -517,7 +517,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_siS_v(const char *who, int src_type, prim_siS_v f, Scheme_Object* g273, int g274, Scheme_Object** g275)
+ void scheme_rtcall_siS_v(const char *who, int src_type, prim_siS_v f, Scheme_Object* g276, int g277, Scheme_Object** g278)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -532,9 +532,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g273;
-    future->arg_i1 = g274;
-    future->arg_S2 = g275;
+      future->arg_s0 = g276;
+    future->arg_i1 = g277;
+    future->arg_S2 = g278;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -544,7 +544,34 @@
   
   
 }
- void* scheme_rtcall_z_p(const char *who, int src_type, prim_z_p f, size_t g276)
+ Scheme_Object* scheme_rtcall_Sii_s(const char *who, int src_type, prim_Sii_s f, Scheme_Object** g279, int g280, int g281)
+   XFORM_SKIP_PROC
+{
+  Scheme_Future_Thread_State *fts = scheme_future_thread_state;
+  future_t *future;
+  double tm;
+  Scheme_Object* retval;
+
+  future = fts->thread->current_ft;
+  future->prim_protocol = SIG_Sii_s;
+  future->prim_func = f;
+  tm = get_future_timestamp();
+  future->time_of_request = tm;
+  future->source_of_request = who;
+  future->source_type = src_type;
+      future->arg_S0 = g279;
+    future->arg_i1 = g280;
+    future->arg_i2 = g281;
+  
+  future_do_runtimecall(fts, (void*)f, 0, 1, 0);
+  fts->thread = scheme_current_thread;
+  future = fts->thread->current_ft;
+  retval = future->retval_s;
+  future->retval_s = 0;
+  receive_special_result(future, retval, 1);
+  return retval;
+}
+ void* scheme_rtcall_z_p(const char *who, int src_type, prim_z_p f, size_t g282)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -559,7 +586,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_z0 = g276;
+      future->arg_z0 = g282;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -569,7 +596,7 @@
   
   return retval;
 }
- Scheme_Object* scheme_rtcall_si_s(const char *who, int src_type, prim_si_s f, Scheme_Object* g277, int g278)
+ Scheme_Object* scheme_rtcall_si_s(const char *who, int src_type, prim_si_s f, Scheme_Object* g283, int g284)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -584,8 +611,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g277;
-    future->arg_i1 = g278;
+      future->arg_s0 = g283;
+    future->arg_i1 = g284;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -595,7 +622,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_sis_v(const char *who, int src_type, prim_sis_v f, Scheme_Object* g279, int g280, Scheme_Object* g281)
+ void scheme_rtcall_sis_v(const char *who, int src_type, prim_sis_v f, Scheme_Object* g285, int g286, Scheme_Object* g287)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -610,9 +637,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g279;
-    future->arg_i1 = g280;
-    future->arg_s2 = g281;
+      future->arg_s0 = g285;
+    future->arg_i1 = g286;
+    future->arg_s2 = g287;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -622,7 +649,7 @@
   
   
 }
- int scheme_rtcall_ss_i(const char *who, int src_type, prim_ss_i f, Scheme_Object* g282, Scheme_Object* g283)
+ int scheme_rtcall_ss_i(const char *who, int src_type, prim_ss_i f, Scheme_Object* g288, Scheme_Object* g289)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -637,8 +664,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g282;
-    future->arg_s1 = g283;
+      future->arg_s0 = g288;
+    future->arg_s1 = g289;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -648,7 +675,7 @@
   
   return retval;
 }
- void scheme_rtcall_iSp_v(const char *who, int src_type, prim_iSp_v f, int g284, Scheme_Object** g285, void* g286)
+ void scheme_rtcall_iSp_v(const char *who, int src_type, prim_iSp_v f, int g290, Scheme_Object** g291, void* g292)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -663,9 +690,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g284;
-    future->arg_S1 = g285;
-    future->arg_p2 = g286;
+      future->arg_i0 = g290;
+    future->arg_S1 = g291;
+    future->arg_p2 = g292;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -675,7 +702,7 @@
   
   
 }
- Scheme_Object* scheme_rtcall_sss_s(const char *who, int src_type, prim_sss_s f, Scheme_Object* g287, Scheme_Object* g288, Scheme_Object* g289)
+ Scheme_Object* scheme_rtcall_sss_s(const char *who, int src_type, prim_sss_s f, Scheme_Object* g293, Scheme_Object* g294, Scheme_Object* g295)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -690,9 +717,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g287;
-    future->arg_s1 = g288;
-    future->arg_s2 = g289;
+      future->arg_s0 = g293;
+    future->arg_s1 = g294;
+    future->arg_s2 = g295;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
@@ -727,7 +754,7 @@
   
   
 }
- void scheme_rtcall_iS_v(const char *who, int src_type, prim_iS_v f, int g290, Scheme_Object** g291)
+ void scheme_rtcall_iS_v(const char *who, int src_type, prim_iS_v f, int g296, Scheme_Object** g297)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -742,8 +769,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g290;
-    future->arg_S1 = g291;
+      future->arg_i0 = g296;
+    future->arg_S1 = g297;
   
   future_do_runtimecall(fts, (void*)f, 0, 1, 0);
   fts->thread = scheme_current_thread;
