@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2006-2015 PLT Design Inc.
+  Copyright (c) 2006-2016 PLT Design Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -676,7 +676,7 @@ void scheme_jit_now(Scheme_Object *f)
 {
   if (SAME_TYPE(SCHEME_TYPE(f), scheme_native_closure_type)) {
     Scheme_Native_Closure *nc;
-    Scheme_Native_Closure_Data *ncd;
+    Scheme_Native_Lambda *ncd;
 
     nc = (Scheme_Native_Closure*)f;
     ncd = nc->code;
@@ -694,7 +694,7 @@ typedef void (*Thread_Start_Child_Proc)(Scheme_Thread *child, Scheme_Object *chi
 void *scheme_module_run_start(Scheme_Env *menv, Scheme_Env *env, Scheme_Object *name)
 {
   Module_Run_Proc proc = (Module_Run_Proc)sjc.module_run_start_code;
-  if (proc)
+  if (proc && !CHECK_RUNSTACK_REGISTER_UPDATE)
     return proc(menv, env, &name);
   else
     return scheme_module_run_finish(menv, env);
@@ -703,7 +703,7 @@ void *scheme_module_run_start(Scheme_Env *menv, Scheme_Env *env, Scheme_Object *
 void *scheme_module_exprun_start(Scheme_Env *menv, int set_ns, Scheme_Object *name)
 {
   Module_Exprun_Proc proc = (Module_Exprun_Proc)sjc.module_exprun_start_code;
-  if (proc)
+  if (proc && !CHECK_RUNSTACK_REGISTER_UPDATE)
     return proc(menv, set_ns, &name);
   else
     return scheme_module_exprun_finish(menv, set_ns);
@@ -712,7 +712,7 @@ void *scheme_module_exprun_start(Scheme_Env *menv, int set_ns, Scheme_Object *na
 void *scheme_module_start_start(struct Start_Module_Args *a, Scheme_Object *name)
 {
   Module_Start_Proc proc = (Module_Start_Proc)sjc.module_start_start_code;
-  if (proc)
+  if (proc && !CHECK_RUNSTACK_REGISTER_UPDATE)
     return proc(a, &name);
   else
     return scheme_module_start_finish(a);
@@ -722,7 +722,7 @@ void scheme_thread_start_child(Scheme_Thread *child, Scheme_Object *child_thunk)
   XFORM_SKIP_PROC
 {
   Thread_Start_Child_Proc proc = (Thread_Start_Child_Proc)sjc.thread_start_child_code;
-  if (proc)
+  if (proc && !CHECK_RUNSTACK_REGISTER_UPDATE)
     proc(child, child_thunk);
   else
     scheme_do_thread_start_child(child, child_thunk);
