@@ -3288,3 +3288,92 @@ intptr_t scheme_count_envbox(Scheme_Object *root, Scheme_Hash_Table *ht)
 }
 
 #endif
+
+/**********************************************************************/
+
+#if RECORD_ALLOCATION_COUNTS
+ 
+/* Allocation profiling --- prints allocated counts (not necessarily
+   still live) after every `NUM_ALLOCS_BEFORE_REPORT` structure and
+   closure allocations. Adjust that constant to match a test program.
+   Also, run with `racket -j` so that structure allocation is not
+   inlined, and don't use places. */
+
+#define NUM_ALLOCS_BEFORE_REPORT 100000
+
+static Scheme_Hash_Table *allocs;
+static int alloc_count;
+static int reporting;
+
+#include "../gc2/my_qsort.c"
+typedef struct alloc_count_result { int pos; int count; } alloc_count_result;
+
+static int smaller_alloc_count(const void *a, const void *b) {
+  return ((alloc_count_result*)a)->count - ((alloc_count_result*)b)->count;
+}
+
+void scheme_record_allocation(Scheme_Object *tag)
+{
+  Scheme_Object *c;
+  
+  if (reporting)
+    return;
+
+  alloc_count++;
+
+  if (!allocs) {
+    REGISTER_SO(allocs);
+    reporting++;
+    allocs = scheme_make_hash_table(SCHEME_hash_ptr);
+    --reporting;
+  }
+
+  c = scheme_hash_get(allocs, tag);
+  if (!c) c = scheme_make_integer(0);
+  scheme_hash_set(allocs, tag, scheme_make_integer(SCHEME_INT_VAL(c)+1));
+    
+  if (alloc_count == NUM_ALLOCS_BEFORE_REPORT) {
+    alloc_count_result *a;
+    int count = allocs->count;
+    int k = 0;
+    int i;
+    char *s;
+
+    reporting++;
+    
+    a = MALLOC_N_ATOMIC(alloc_count_result, count);
+    printf("\n");
+    for (i = allocs->size; i--; ) {
+      if (allocs->vals[i]) {
+        a[k].pos = i;
+        a[k].count = SCHEME_INT_VAL(allocs->vals[i]);
+        k++;
+      }
+    }
+    my_qsort(a, allocs->count, sizeof(alloc_count_result), smaller_alloc_count);
+    
+    for (i = 0; i < count; i++) {
+      tag = allocs->keys[a[i].pos];
+
+      if (SCHEME_INTP(tag)) {
+        s = scheme_get_type_name(SCHEME_INT_VAL(tag));
+      } else {
+        if (SAME_TYPE(SCHEME_TYPE(tag), scheme_lambda_type)
+            && ((Scheme_Lambda *)tag)->name)
+          tag = ((Scheme_Lambda*)tag)->name;
+        else if (SAME_TYPE(SCHEME_TYPE(tag), scheme_case_lambda_sequence_type)
+                 && ((Scheme_Case_Lambda *)tag)->name)
+          tag = ((Scheme_Case_Lambda*)tag)->name;
+      
+        s = scheme_write_to_string(tag, NULL);
+      }
+      
+      printf("%d %s\n", a[i].count, s);
+    }
+    
+    alloc_count = 0;
+    --reporting;
+  }
+}
+
+#endif
